package internal

import (
	"encoding/json"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"html"
	"plugins/alipay_redpack/internal/po"
	"plugins/alipay_redpack/internal/vo"
	"plugins/utils/alipay"
	"time"
)

type Config struct {
	AppId   string `json:"app_id"`
	Prk     string `json:"prk"` // 私钥
	PukPath string `json:"npk"` // 验签公钥

	MchCertPath  string `json:"mch_cert_path"`
	RootCertPath string `json:"root_cert_path"`
}

func transConfig(config []byte) (*Config, error) {
	var c Config
	err := json.Unmarshal(config, &c)
	if err != nil {
		return nil, err
	}
	//dir, _ := os.Getwd()
	//filePath := fmt.Sprintf("%s/%s/%s/%s", dir, "cert", "alipay", c.AppId)
	//if !fileExists(filePath) {
	//	return nil, fmt.Errorf("授权文件信息不存在,请联系技术人员处理")
	//}
	/**
	AppId:        a.bc.Alipay.AppId,
	Prk:          a.bc.Alipay.GetPrk(),
	PukPath:      fmt.Sprintf("%s/%s", filePath, "alipayCertPublicKey_RSA2.crt"),
	MchCertPath:  fmt.Sprintf("%s/%s", filePath, a.bc.Alipay.MchCertName),
	RootCertPath: fmt.Sprintf("%s/%s", filePath, "alipayRootCert.crt"),
	*/
	return &c, nil
}

func (c *Config) paramReq(req po.Req, method string) (*po.Param, error) {
	if err := req.Validate(); err != nil {
		return nil, err
	}
	cert, err := alipay.GetCert(c.MchCertPath, c.RootCertPath, c.PukPath, c.AppId)
	if err != nil {
		return nil, err
	}
	return &po.Param{
		AlipayRootCertSn: cert.RootCertSN,
		AppCertSn:        cert.MchCertSN,
		AppId:            c.AppId,
		Method:           method,
		Format:           vo.Format,
		Charset:          vo.Charset,
		SignType:         vo.SignType,
		Timestamp:        time.Now().Format(time.DateTime),
		Version:          vo.Version,
		BizContent:       string(req.ToJson()),
		Sign:             "",
	}, nil
}

func orderReq(order *proto.OrderRequest_Order, product *proto.OrderRequest_Product) (*po.OrderReq, error) {
	type ProductExtra struct {
		Wishing string `json:"wishing"`
	}
	var productExtra ProductExtra
	if product.Extra == nil {
		return nil, fmt.Errorf("product extra is nil")
	}
	err := json.Unmarshal(product.Extra, &productExtra)
	if err != nil {
		return nil, fmt.Errorf("product extra json unmarshal error: %v", err)
	}
	payeeInfo := po.PayeeInfo{
		Identity:     order.Account,
		IdentityType: "ALIPAY_USER_ID",
	}
	if isValidPhoneNumber(order.Account) || isEmailValid(order.Account) {
		type OrderExtra struct {
			Name string `json:"name"`
		}
		if order.Extra == nil {
			return nil, fmt.Errorf("order extra is nil")
		}
		var orderExtra OrderExtra
		err = json.Unmarshal(order.Extra, &orderExtra)
		if err != nil {
			return nil, fmt.Errorf("order extra json unmarshal error: %v", err)
		}
		if orderExtra.Name == "" {
			return nil, fmt.Errorf("姓名，当identity_type=ALIPAY_LOGON_ID时，本字段必填")
		}
		payeeInfo.IdentityType = "ALIPAY_LOGON_ID"
		payeeInfo.Name = orderExtra.Name
	}

	o := &po.OrderReq{
		OutBizNo:       order.OrderNo,
		TransAmount:    order.Amount,
		ProductCode:    "STD_RED_PACKET",
		BizScene:       "DIRECT_TRANSFER",
		OrderTitle:     productExtra.Wishing,
		Remark:         productExtra.Wishing,
		PayeeInfo:      &payeeInfo,
		BusinessParams: `{"sub_biz_scene":"REDPACKET"}`,
	}
	return o, nil
}

func orderResp(request *proto.OrderRequest, resp po.OrderResp) *proto.OrderResponse {
	data, _ := json.Marshal(resp)
	return &proto.OrderResponse{
		Result: &proto.Result{
			Status:  resp.GetOrderStatus(),
			OrderNo: request.Order.OrderNo,
			TradeNo: resp.Response.OrderId,
			Message: resp.GetMsg(),
			Data:    data,
		},
	}
}

func queryReq(in *proto.QueryRequest_Order) (*po.QueryReq, error) {
	return &po.QueryReq{
		OutBizNo:    in.OrderNo,
		OrderId:     in.TradeNo,
		ProductCode: "STD_RED_PACKET",
		BizScene:    "DIRECT_TRANSFER",
	}, nil
}

func queryResp(request *proto.QueryRequest, resp po.QueryResp) *proto.QueryResponse {
	data, _ := json.Marshal(resp)
	return &proto.QueryResponse{
		Result: &proto.Result{
			Status:  resp.Data.Status.GetOrderStatus(),
			OrderNo: request.Order.OrderNo,
			TradeNo: request.Order.TradeNo,
			Message: resp.Data.Msg,
			Data:    data,
		},
	}
}

func notifyReq(in *proto.NotifyRequest) *po.Notify {
	var n *po.Notify
	_ = json.Unmarshal(in.Queries, &n)
	n.BizContent = html.UnescapeString(n.BizContent)
	return n
}

func notifyResp(n *po.Notify) *proto.NotifyResponse {
	var b po.NotifyBizContent
	_ = json.Unmarshal([]byte(n.BizContent), &b)
	return &proto.NotifyResponse{
		Result: &proto.Result{
			Status:  b.Status.GetOrderStatus(),
			OrderNo: b.OutBizNo,
			TradeNo: b.OrderId,
			Message: b.Status.GetText(),
		},
	}
}
